from fti.file_types import FileTypes


FILE_MIMES: dict[FileTypes, set[str]] = {
    FileTypes._3G2: {"video/3gpp2"},
    FileTypes._3GP: {"video/3gpp"},
    FileTypes._7Z: {"application/x-7z-compressed"},
    FileTypes.AIF: {"audio/x-aiff"},
    FileTypes.AIFC: {"audio/x-aiff"},
    FileTypes.AIFF: {"audio/x-aiff"},
    FileTypes.APK: {"application/vnd.android.package-archive"},
    FileTypes.ASF: {"video/x-ms-asf"},
    FileTypes.AVI: {"video/x-msvideo"},
    FileTypes.BIN: {"application/octet-stream"},
    FileTypes.BMP: {"image/bmp"},
    FileTypes.BZ2: {"application/x-bzip2"},
    FileTypes.CAB: {"application/vnd.ms-cab-compressed"},
    FileTypes.CLASS: {"application/java-vm"},
    FileTypes.CSS: {"text/css"},
    FileTypes.DEB: {"application/x-debian-package"},
    FileTypes.DER: {"application/x-x509-ca-cert"},
    FileTypes.DJV: {"image/vnd.djvu"},
    FileTypes.DJVU: {"image/vnd.djvu"},
    FileTypes.DMG: {"application/x-apple-diskimage"},
    FileTypes.DOC: {"application/msword"},
    FileTypes.DOCX: {"application/vnd.openxmlformats-officedocument.wordprocessingml.document"},
    FileTypes.EXE: {"application/x-msdownload"},
    FileTypes.FLAC: {"audio/x-flac"},
    FileTypes.GIF: {"image/gif"},
    FileTypes.ICO: {"image/x-icon"},
    FileTypes.ISO: {"application/x-iso9660-image"},
    FileTypes.JAR: {"application/java-archive"},
    FileTypes.JPEG: {"image/jpeg"},
    FileTypes.JPG: {"image/jpeg"},
    FileTypes.MID: {"audio/midi"},
    FileTypes.MIDI: {"audio/midi"},
    FileTypes.MK3D: {"video/x-matroska"},
    FileTypes.MKA: {"audio/x-matroska"},
    FileTypes.MKS: {"video/x-matroska"},
    FileTypes.MKV: {"video/x-matroska"},
    FileTypes.MP3: {"audio/mpeg"},
    FileTypes.MUS: {"application/vnd.musician"},
    FileTypes.ODP: {"application/vnd.oasis.opendocument.presentation"},
    FileTypes.ODS: {"application/vnd.oasis.opendocument.spreadsheet"},
    FileTypes.ODT: {"application/vnd.oasis.opendocument.text"},
    FileTypes.OGA: {"audio/ogg"},
    FileTypes.OGG: {"audio/ogg"},
    FileTypes.OGV: {"video/ogg"},
    FileTypes.PDB: {"application/vnd.palm"},
    FileTypes.PDF: {"application/pdf"},
    FileTypes.PIC: {"image/x-pict"},
    FileTypes.PNG: {"image/png"},
    FileTypes.PPT: {"application/vnd.ms-powerpoint"},
    FileTypes.PPTX: {"application/vnd.openxmlformats-officedocument.presentationml.presentation"},
    FileTypes.PS: {"application/postscript"},
    FileTypes.PSD: {"image/vnd.adobe.photoshop"},
    FileTypes.RAR: {"application/x-rar-compressed"},
    FileTypes.SND: {"audio/basic"},
    FileTypes.SWF: {"application/x-shockwave-flash"},
    FileTypes.TAR: {"application/x-tar"},
    FileTypes.TIF: {"image/tiff"},
    FileTypes.TIFF: {"image/tiff"},
    FileTypes.TXT: {"text/plain"},
    FileTypes.WAV: {"audio/x-wav"},
    FileTypes.WEBM: {"video/webm"},
    FileTypes.WEBP: {"image/webp"},
    FileTypes.WMA: {"audio/x-ms-wma"},
    FileTypes.WMV: {"video/x-ms-wmv"},
    FileTypes.WOFF: {"application/font-woff"},
    FileTypes.XAR: {"application/vnd.xara"},
    FileTypes.XLS: {"application/vnd.ms-excel"},
    FileTypes.XLSX: {"application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"},
    FileTypes.XML: {"application/xml"},
    FileTypes.ZIP: {"application/zip"},
}


def _check_mime_file_type(mime: str, file_type: FileTypes) -> bool:
    mimes = FILE_MIMES.get(file_type)

    assert mimes

    return mime in mimes


def get_file_types_by_mime(mime: str) -> set[FileTypes]:
    result = set()

    for part in mime.split(" "):
        t_part = part.replace(";", "")
        for file_type in FILE_MIMES:
            if _check_mime_file_type(t_part, file_type):
                result.add(file_type)

    return result



from .get_by_content_disposition import get_file_types_by_content_disposition
from .get_by_content_header import get_file_types_by_content
from .get_by_mimetype import get_file_types_by_mime
from .get_by_url import get_file_types_by_url



import httpx

from . import FileTypes
from .file_type_getters import (
    get_file_types_by_content,
    get_file_types_by_content_disposition,
    get_file_types_by_mime,
    get_file_types_by_url,
)


def get_file_types(url: str) -> set[FileTypes]:
    if result := get_file_types_by_url(url):
        return result

    try:
        with httpx.stream("GET", url, follow_redirects=True) as response:
            content_disposition = response.headers.get("content-disposition")
            if content_disposition and (result := get_file_types_by_content_disposition(content_disposition)):
                return result

            content_type = response.headers.get("content-type")
            if content_type and (result := get_file_types_by_mime(content_type)):
                return result

            content = next(response.iter_bytes(64))

            if result := get_file_types_by_content(content):
                return result
    except httpx.UnsupportedProtocol:
        raise ValueError("Wrong url!")

    return set()


async def get_file_types_async(url: str) -> set[FileTypes]:
    if result := get_file_types_by_url(url):
        return result

    client = httpx.AsyncClient()
    try:
        async with client.stream("GET", url, follow_redirects=True) as response:
            content_disposition = response.headers.get("content-disposition")
            if content_disposition and (result := get_file_types_by_content_disposition(content_disposition)):
                return result

            content_type = response.headers.get("content-type")
            if content_type and (result := get_file_types_by_mime(content_type)):
                return result

            content = await response.aiter_bytes(64).__anext__()

            if result := get_file_types_by_content(content):
                return result
    except httpx.UnsupportedProtocol:
        raise ValueError("Wrong url!")
    finally:
        await client.aclose()

    return set()



from enum import Enum


class FileTypes(Enum):
    _3G2 = "3g2"
    _3GP = "3gp"
    _7Z = "7z"
    _8SV = "8sv"
    _8SVX = "8svx"
    ACBM = "acbm"
    AIF = "aif"
    AIFC = "aifc"
    AIFF = "aiff"
    ANBM = "anbm"
    ANIM = "anim"
    APK = "apk"
    ASF = "asf"
    AVI = "avi"
    BAC = "bac"
    BIN = "bin"
    BMP = "bmp"
    BPG = "bpg"
    BZ2 = "bz2"
    CAB = "cab"
    CIN = "cin"
    CLASS = "class"
    CMUS = "cmus"
    CR2 = "cr2"
    CRX = "crx"
    CSS = "css"
    CWK = "cwk"
    DAT = "dat"
    DBA = "dba"
    DCM = "dcm"
    DEB = "deb"
    DER = "der"
    DEX = "dex"
    DIB = "dib"
    DJV = "djv"
    DJVU = "djvu"
    DMG = "dmg"
    DOC = "doc"
    DOCX = "docx"
    DPX = "dpx"
    EXE = "exe"
    EXR = "exr"
    FAXX = "faxx"
    FH8 = "fh8"
    FITS = "fits"
    FLAC = "flac"
    FLIF = "flif"
    FTXT = "ftxt"
    GIF = "gif"
    GZ = "gz"
    IBM = "ibm"
    ICO = "ico"
    IDX = "idx"
    IFF = "iff"
    ILBM = "ilbm"
    ISO = "iso"
    JAR = "jar"
    JPEG = "jpeg"
    JPG = "jpg"
    LBM = "lbm"
    LEP = "lep"
    LZ = "lz"
    LZ4 = "lz4"
    MID = "mid"
    MIDI = "midi"
    MK3D = "mk3d"
    MKA = "mka"
    MKS = "mks"
    MKV = "mkv"
    MLV = "mlv"
    MP3 = "mp3"
    MSG = "msg"
    MUS = "mus"
    NES = "nes"
    ODP = "odp"
    ODS = "ods"
    ODT = "odt"
    OGA = "oga"
    OGG = "ogg"
    OGV = "ogv"
    PDB = "pdb"
    PDF = "pdf"
    PIC = "pic"
    PIF = "pif"
    PNG = "png"
    PPT = "ppt"
    PPTX = "pptx"
    PS = "ps"
    PSD = "psd"
    RAR = "rar"
    RPM = "rpm"
    SEA = "sea"
    SMU = "smu"
    SMUS = "smus"
    SND = "snd"
    STG = "stg"
    SVX = "svx"
    SWF = "swf"
    TAR = "tar"
    TAR_GZ = "tar_gz"
    TAR_Z = "tar_z"
    TDA = "tda"
    TIF = "tif"
    TIFF = "tiff"
    TOAST = "toast"
    TOX = "tox"
    TXT = "txt"
    VMDK = "vmdk"
    VSDX = "vsdx"
    WASM = "wasm"
    WAV = "wav"
    WEBM = "webm"
    WEBP = "webp"
    WMA = "wma"
    WMV = "wmv"
    WOFF = "woff"
    WOFF2 = "woff2"
    XAR = "xar"
    XLS = "xls"
    XLSX = "xlsx"
    XML = "xml"
    YTR = "ytr"
    YUV = "yuv"
    YUVN = "yuvn"
    Z = "z"
    ZIP = "zip"



import os
from urllib.parse import urlparse

from fti.file_types import FileTypes


def get_file_types_by_url(url: str) -> set[FileTypes]:
    parsed_path = urlparse(url.lower())
    filename = os.path.basename(parsed_path.path)

    if "." not in filename:
        return set()

    file_format = filename.split(".")[-1]

    try:
        return {FileTypes(file_format)}
    except ValueError:
        return set()



from fti.file_types import FileTypes


def get_file_types_by_content_disposition(content_disposition: str) -> set[FileTypes]:
    filename = None

    for part in content_disposition.split(" "):
        if part.lower().startswith("filename="):
            filename = part.lower().replace("filename=", "").replace('"', "").replace(";", "")
            break

    if filename is None:
        return set()

    if "." not in filename:
        return set()

    file_format = filename.split(".")[-1]

    try:
        return {FileTypes(file_format)}
    except ValueError:
        return set()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fti', 'fti.file_type_getters']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.22.0,<0.23.0']

setup_kwargs = {
    'name': 'file-type-identifier',
    'version': '0.1.1',
    'description': 'Library for identifying file type',
    'long_description': '# file-type-indentifier\n\nLibrary for identifying file type\n\n## Examples\n\n```\nimport asyncio\n\nfrom fti import FileTypes, get_file_types, get_file_types_async\n\n\n# sync\nfile_types = get_file_types("http://example.com/download_pdf")\n\nFileTypes.PDF in file_types\n\n\n# async\nfile_types = asyncio.run(\n    get_file_types_async("http://example.com/download_pdf")\n)\n\nFileTypes.PDF in file_types\n```',
    'author': 'Kurbanov Bulat',
    'author_email': 'b.kurbanov@speechki.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
